package br.ufrn.imd.client.utils;

import br.ufrn.imd.client.domain.Client;
import br.ufrn.imd.client.exception.BadRequestException;
import br.ufrn.imd.client.utils.message.CredentialsMessage;
import lombok.Data;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestTemplate;

import java.util.Scanner;

import static ufrn.imd.utils.NameSpace.*;

@Data
@Component
public class MachineState {

    private RestTemplate restTemplate = new RestTemplate();
    public static final String INITIAL= "INITIAL";
    public static final String READ = "READ";
    public static final String DEPOSIT = "DEPOSIT";
    public static final String BALANCE = "BALANCE";
    public static final String WITHDRAWAL = "WITHDRAWAL";
    public static final String TRANSFER = "TRANSFER";

    public static final String START = "START";
    public static final String INVALID_INPUT = "INVALID_INPUT";
    public static final String QUIT = "QUIT";

    private String state;
    private Scanner sc;
    private String inputValue;

    private Client client;

    public MachineState() {
        state = INITIAL;
        inputValue = new String();
        sc = new Scanner(System.in);
    }

    public void login() {
        System.out.println(">>> Please enter your bank account number");
        String accountNumber = sc.nextLine();

        System.out.println(">>> Please enter your password");
        String password = sc.nextLine();

        try {
            ResponseEntity<Client> response = restTemplate.postForEntity(
                    HOST_AUTHENTICATOR_SERVICE,
                    new CredentialsMessage(accountNumber, password),
                    Client.class);
            if(response.getStatusCodeValue() == 200) {
                state = INITIAL;
                this.client = response.getBody();
            }
        } catch (HttpClientErrorException ex){
            System.out.println(">>> Bad credentials");
        }


        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

    public Boolean hasUser() {
        return client != null;
    }

    public Boolean quit() {
        return !state.equals(QUIT);
    }

    public void update() {
        if(state.equals(INITIAL)) {
            state = START;
        } else if(state.equals(START)) {
            state = READ;
        } else if(state.equals(READ)) {
            state = inputValue;
        } else if(state.equals(INVALID_INPUT)) {
            state = START;
        } else if(state.equals(DEPOSIT)) {
            state = START;
        } else if(state.equals(BALANCE)) {
            state = START;
        } else if(state.equals(WITHDRAWAL)) {
            state = START;
        } else if(state.equals(TRANSFER)) {
            state = START;
        }
    }

    public void processEvents()  {
        if(state.equals(READ)) {
            inputValue = sc.nextLine().trim().toUpperCase();
            if(!inputValue.isEmpty() &&
                    !inputValue.equals(QUIT) &&
                    !inputValue.equals(DEPOSIT) &&
                    !inputValue.equals(BALANCE) &&
                    !inputValue.equals(WITHDRAWAL) &&
                    !inputValue.equals(TRANSFER)) {
                inputValue = INVALID_INPUT;
            }
        } else if(state.equals(DEPOSIT)) {
            String v = sc.nextLine();
            try {
                ResponseEntity<Client> response =
                        restTemplate.getForEntity(
                                HOST_DEPOSIT_SERVICE +
                                        String.format("/%d?value=%s", this.client.getId(), v),
                                Client.class);
                if (response.getStatusCodeValue() == 200) {
                    System.out.println(
                            String.format(">>> Your balance is: %.2f", response.getBody().getAccount().getBalance())
                    );
                }
            } catch (Exception e) {
                System.out.println("Invalid operation!");
            }
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }

        } else if(state.equals(BALANCE)) {
            try {
                ResponseEntity<Client> response =
                        restTemplate.getForEntity(HOST_BALANCE_SERVICE +
                                String.format("/%d", this.client.getId()), Client.class);
                if (response.getStatusCodeValue() == 200) {
                    System.out.println(
                            String.format(">>> Your balance is: %.2f", response.getBody().getAccount().getBalance())
                    );
                }
            } catch (Exception e) {
                System.out.println("Invalid operation!");
            }
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }

        } else if(state.equals(WITHDRAWAL)) {
            String v = sc.nextLine();
            try {
                ResponseEntity<Client> response =
                        restTemplate.getForEntity(
                                HOST_WITHDRAWAL_SERVICE +
                                        String.format("/%d?value=%s", this.client.getId(), v),
                                Client.class);
                if (response.getStatusCodeValue() == 200) {
                    System.out.println(
                            String.format(">>> Your balance is: %.2f", response.getBody().getAccount().getBalance())
                    );
                } else if (response.getStatusCodeValue() == 500) {
                    System.out.println(
                            String.format(">>> %s", response.getBody())
                    );
                }
            } catch (Exception e) {
                System.out.println("Invalid operation!");
            }
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        } else if(state.equals(TRANSFER)) {
            String accountNumber = sc.nextLine();
            System.out.println(">>> Please enter with the value that you want transfer");
            String v = sc.nextLine();
            try {
                ResponseEntity<Client> response =
                        restTemplate.getForEntity(
                                HOST_TRANSFER_SERVICE +
                                        String.format("/%d?value=%s&toAccount=%s",
                                                this.client.getId(),
                                                v,
                                                accountNumber
                                        ),
                                Client.class);

                if (response.getStatusCodeValue() == 200) {
                    System.out.println(
                            String.format(">>> Your balance is: %.2f", response.getBody().getAccount().getBalance())
                    );
                } else if (response.getStatusCodeValue() == 500) {
                    System.out.println(
                            String.format(">>> %s", response.getBody())
                    );
                }
            } catch (Exception e) {
                System.out.println("Invalid operation!");
            }
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        } else if(state.equals(QUIT)) {
            this.client = null;
            System.out.println(">>> You have logged out");
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
    }

    public void render() {
        try {
            if(state.equals(INITIAL)) {
                System.out.println(">>> Initializing...");
                Thread.sleep(1000);
            } else if(state.equals(START)) {
                System.out.println("These are the avaliable operations");
                System.out.println("---------------------------------");
                System.out.println(" - deposit \t\t - balance");
                System.out.println(" - withdrawal \t - transfer");
                System.out.println(" - quit");
                System.out.println("---------------------------------");
                Thread.sleep(1000);
            } else if(state.equals(READ)) {
                System.out.println(">>> Please enter with wich operation do you want");
            } else if(state.equals(INVALID_INPUT)) {
                System.out.println(">>> Please enter with a VALID operation");
                Thread.sleep(3000);
            } else if(state.equals(DEPOSIT)) {
                System.out.println(">>> Starting deposit operation");
                System.out.println(">>> Please enter with the value that you want deposit");
            } else if(state.equals(WITHDRAWAL)) {
                System.out.println(">>> Starting withdrawal operation");
                System.out.println(">>> Please enter the value you want to withdraw");
            } else if(state.equals(TRANSFER)) {
                System.out.println(">>> Starting transfer operation");
                System.out.println(">>> Please enter with the recipient's account number");
            }
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
    }

}
